@@ -82,10 +82,17 @@ few special cases:
     PyScons(targets, sources, "my_scons.py", clean = True) # intermediate file deleted
     PyScons(targets, sources, "my_scons.py", clean = False) # intermediate file retained
 
-Unfortunately, dependency tracking does not propagate up from the spawned scons. So 
-sources that are not explicitely included in the "sources" variable will not be tracked. There 
-may be no good way around this, short of creating a new scanner and file extension (which I 
-may do if there interest in this feature).
+Unfortunately, dependency tracking does not propagate up from the spawned scons. In this example, 
+"step1.py" and "step2.py" will not be tracked and changes to them will not trigger a rebuild. There
+is a trick around this, add the following two lines to "my_scons.py"
+
+    ### step1.py
+    #DEPENDS step2.py
+
+These two comments illustrate the two ways of explicetely including the dependency on the two 
+scripts used on the scons file. To help distinguish files which are to be run in this ways 
+(being called by PyScons), they may be given the extensions ".scons" or ".pyscons" as well. 
+In this example, this would amount to renaming "my_scons.py" to "my_scons.scons" 
 
 PyScanner
 ---------
@@ -121,13 +128,17 @@ S. Joshua Swamidass (homepage_)
 .. _homepage: http://swami.wustl.edu/
 .. _Scons: http://www.scons.org/
 """
-
 import os
 from SCons import Util
 from SCons.Scanner import Scanner
 from SCons.Script.SConscript import DefaultEnvironmentCall
 from modulefinder import ModuleFinder as MF
-from SCons.Script import ARGUMENTS, DefaultEnvironment
+from SCons.Script import *
+import re
+import hashlib 
+import tempfile
+import base64
+from itertools import chain
 
 class PyScanner(MF):
     """
@@ -150,6 +161,16 @@ class PyScanner(MF):
         except: pass
 
         imports = [m.__file__ for m in self.modules.values() if m.__file__ != None]
+
+        if str(node).split('.')[-1] in ["py", "scons", "pyscons"]:
+            contents = open(str(node)).read()
+            D = re.findall("#"+"##(.+)", contents) + re.findall("#" +"DEPENDS(.+)", contents, re.I)
+            D = [f for f in chain(*(f.split() for f in D))]
+            d = os.path.dirname(str(node))
+            for f in D:
+                f = os.path.join(d,f)
+                if os.path.exists(f): imports.append(f)
+
         imports = list(os.path.abspath(m) for m in imports if self.path_filter(m))
         return imports
 
@@ -186,17 +207,32 @@ def _PyScript(env, target, source, command = "python $SOURCES",
     command = [ env.subst(c, target=target, source=subst) for c in command ]
     return env.Command(target, source, command, *args, **kwargs)  
 
-def _PyScons(env, target, source, scons, options="-Q", clean = True):
+def _PyScons(env, target, source, scons, options="-Q", clean = True, tempdir = None):
     target = ToList(target) 
     source = ToList(source) 
 
     args = " ".join(["in%g=%s" % (n+1, str(s)) for n,s in enumerate(source)])
     args += " " + " ".join(["out%g=%s" % (n+1, str(s)) for n,s in enumerate(target)])
 
-    if env.GetOption("clean") and not clean: env.Execute("scons %s -f%s %s --clean" % (options, scons, args))
+
+    cmd = "scons %s -f%s %s --clean" % (options, scons, args)
+    if tempdir == None:
+        m = hashlib.sha1()
+        m.update(cmd)
+        TEMP = os.path.join("temp", base64.b64encode(m.digest(), "+="), '')
+    else: TEMP = tempdir
+
+    args += " " + "temp=" + TEMP    
+
+    if env.GetOption("clean") and not clean: 
+        env.Execute("scons %s -f%s %s --clean" % (options, scons, args))
+        env.Execute(Delete(TEMP))
 
     cmd = "scons %s -f${SOURCES[-1]} %s" % (options, args)
-    if clean: cmd = [cmd, "scons %s -f${SOURCES[-1]} %s --clean" % (options, args)]
+    if clean: 
+        cmd = [Mkdir(TEMP), cmd, "scons %s -f${SOURCES[-1]} %s --clean" % (options, args), Delete(TEMP)]
+    else:
+        cmd = [Mkdir(TEMP), cmd]
 
     return env.Command(target, source + [scons], cmd)
 
@@ -239,12 +275,8 @@ def PySconsSetup(args=ARGUMENTS, tools=['default', PYTOOL()]):
 
     env.NoClean(targets)
     
-    try: basename = args["base"] 
+    try: basename = args["temp"] 
     except: basename = None
 
-    if basename == None: 
-        try: basename = targets[-1]
-        except: basename = "TEMP"
-
     return targets, sources, basename
 
